# I will try and make this quality checks file as standard as possible
# so that it can be used in a lot of different scenarios
# It will take in the directory it needs to check and then perform multiple different 
# checks on the data.

import sys
import os
import argparse
import pandas as pd
import xarray as xr
import numpy as np
import netCDF4 as nc4
from netCDF4 import Dataset as nc
from io import StringIO
from quality_checks import Checks
from datetime import datetime, timedelta
import paramiko 


class AeronetChecks(Checks):
    """
    Checks specific to Aeronet, this could possibly go in a separate file 
    """
    def __init__(self, indir, file):
        super().__init__(indir, file)
        self.dataset = "Aeronet"

    def check_number_of_stations(self, number_of_stations=50):
        """
        This will mainly be for the aeronet download so that we can check that
        all possible stations have been downloaded for that day
        """
        print("Checking number of stations")
        columns = ["AERONET_Site"]

        with open(self.filepath) as htmlfile:
            csvdata = StringIO((''.join(htmlfile.readlines()[7:-1]).replace('<br>', '')))
            
        df = pd.read_csv(csvdata, engine='python', header=0,
                        usecols=columns,
                        index_col=False,)

        if df["AERONET_Site"].nunique() < number_of_stations:
            return "Number of stations not consistent with what we would expect - there are less than {}".format(number_of_stations)
    
    def aeronet_original_files_check(self):
        error_list = []
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:

            file_size = self.check_file_size()
            if file_size != None:
                error_list.append(file_size)
    
            stations = self.check_number_of_stations()
            if stations != None:
                error_list.append(stations)
    
            if error_list != []:
                self.send_email_over_ssh(error_list)
            else:
                print("No errors reported")


    def aeronet_formatted_files_check(self):
        """
        Checking formatted Aeronet files at 
        """
        error_list = []
        # First check that the file exists
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:
            # If the file exist next check that it hasn't been corrupted
            file_corrupted = self.check_corrupted_netcdf()
            if file_corrupted != None:
                error_list.append(file_corrupted)
                self.send_email_over_ssh(error_list)
            else:
                #If not corrupted continue with further checks
                file_size = self.check_file_size()
                if file_size != None:
                    error_list.append(file_size)
    
                if error_list != []:
                    print(error_list)
                else:
                    print("No errors reported")



if __name__ == "__main__":
    print(sys.argv)

    #options = getOptions(sys.argv[1:])
    today = datetime.today()
    yesterday = (today - timedelta(days=1)).strftime("%Y%m%d")
    indir = "/esarchive/obs/nasa-aeronet/directsun_v3-lev15/original_files"#options.indir
    file = "{}.html".format(yesterday)
    aer_ori = AeronetChecks(indir, file)
    aer_ori.aeronet_original_files_check()

    # 3hourly test
    # Formatted file check
    date = today.strftime("%Y%m")
    path = "/esarchive/obs/nasa-aeronet/directsun_v3-lev15/3hourly"
    elements = ["ae440-870aero", "od550aero"]
    for element in elements:
        indir = path + "/{}".format(element)
        file = "{}_{}.nc".format(element, date)
        print("Checking file {}".format(file))
        aer_fmt = AeronetChecks(indir, file)
        aer_fmt.aeronet_formatted_files_check()

    # hourly test
    # Formatted file check
    date = today.strftime("%Y%m")
    path = "/esarchive/obs/nasa-aeronet/directsun_v3-lev15/hourly"
    elements = ["ae440-870aero", "od550aero"]
    for element in elements:
        indir = path + "/{}".format(element)
        file = "{}_{}.nc".format(element, date)
        print("Checking file {}".format(file))
        aer_fmt = AeronetChecks(indir, file)
        aer_fmt.aeronet_formatted_files_check()
    
    sys.exit()