# I will try and make this quality checks file as standard as possible
# so that it can be used in a lot of different scenarios
# It will take in the directory it needs to check and then perform multiple different 
# checks on the data.

import sys
import os
import argparse
import pandas as pd
import xarray as xr
import numpy as np
import netCDF4 as nc4
from netCDF4 import Dataset as nc
from io import StringIO
from quality_checks import Checks
from datetime import datetime, timedelta
import paramiko


class EionetChecks(Checks):
    """
    Checks specific to Eionet, this could possibly go in a separate file 
    """
    def __init__(self, indir, file):
        super().__init__(indir, file)
        self.dataset = "Eionet"
        

    def check_specific_caliope_stations(self):
        print("Checking Eionet Barcelona stations")
        bcn_stations_list = ['ES1983A', 'ES1929A', 'ES0692A', 'ES1396A',
                     'ES1992A', 'ES1438A', 'ES1480A', 'ES1679A',
                     'ES0691A', 'ES1856A', 'ES1148A', 'ES1438A']
        print(self.filepath)
        df = pd.read_csv(self.filepath, engine='python', header=0, encoding= 'unicode_escape')
        stations_missing = []
        for station in bcn_stations_list:
            # Checking station is in  original eionet file
            if not df.station_code.isin([station]).any():
                print("{} station not there".format(station))
                stations_missing.append(station)

        if stations_missing:
            return "Original Eionet csv is missing the following stations {}".format(stations_missing)

    def eionet_original_files_check(self):
        error_list = []
        file_exists = self.check_that_file_exists()
        print(self.filepath)
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:

            file_size = self.check_file_size()
            if file_size != None:
                error_list.append(file_size)
    
            stations = self.check_specific_caliope_stations()
            if stations != None:
                error_list.append(stations)
    
            if error_list != []:
                self.send_email_over_ssh(error_list)
            else:
                print("No errors reported")


    def eionet_formatted_files_check(self):
        """
        Checking formatted Aeronet files at 
        """
        error_list = []
        # First check that the file exists
        file_exists = self.check_that_file_exists()
        if file_exists != None:
            error_list.append(file_exists)
            self.send_email_over_ssh(error_list)
        else:
            # If the file exist next check that it hasn't been corrupted
            file_corrupted = self.check_corrupted_netcdf()
            if file_corrupted != None:
                error_list.append(file_corrupted)
                self.send_email_over_ssh(error_list)
            else:
                #If not corrupted continue with further checks
                file_size = self.check_file_size()
                if file_size != None:
                    error_list.append(file_size)
    
                if error_list != []:
                    print(error_list)
                else:
                    print("No errors reported")



if __name__ == "__main__":
    print(sys.argv)

    # Original files check
    #options = getOptions(sys.argv[1:])
    today = datetime.today()
    yesterday = (today - timedelta(days=1)).strftime("%Y%m%d")
    indir = "/esarchive/obs/eea/eionet/original_files"#options.indir
    file = "{}.csv".format(yesterday)
    eio_ori = EionetChecks(indir, file)
    eio_ori.eionet_original_files_check()

    # Formatted file check
    date = today.strftime("%Y%m")
    path = "/esarchive/obs/eea/eionet/hourly"
    elements = ["pm10", "pm2p5", "sconcco", "sconcno", "sconcno2", "sconco3", "sconcso2"]
    for element in elements:
        indir = path + "/{}".format(element)
        file = "{}_{}.nc".format(element, date)
        print("Checking file {}".format(file))
        eio_fmt = EionetChecks(indir, file)
        eio_fmt.eionet_formatted_files_check()
    
    sys.exit()


